<?php

declare(strict_types=1);

namespace Drupal\migrate_dc\Plugin\migrate\process;

use Drupal\Component\Uuid\UuidInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\migrate\Attribute\MigrateProcess;
use Drupal\migrate\MigrateExecutableInterface;
use Drupal\migrate\ProcessPluginBase;
use Drupal\migrate\Row as MigrateRow;
use Symfony\Component\DependencyInjection\ContainerInterface;

#[MigrateProcess(
  id: 'migrate_dc_ensure_uuid',
)]
class MigrateDcEnsureUuid extends ProcessPluginBase implements ContainerFactoryPluginInterface {

  protected string $uuidPattern = '/^[a-z0-9]{8}-[a-z0-9]{4}-[a-z0-9]{4}-[a-z0-9]{4}-[a-z0-9]{12}$/i';

  /**
   * {@inheritDoc}
   *
   * @phpstan-param array<string, mixed> $configuration
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition,
  ): static {
    // @phpstan-ignore-next-line
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('uuid'),
    );
  }

  /**
   * {@inheritdoc}
   *
   * @phpstan-param array<string, mixed> $configuration
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    protected UuidInterface $uuidGenerator,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public function transform($value, MigrateExecutableInterface $migrate_executable, MigrateRow $row, $destination_property) {
    return $this->isUuid((string) $value)
      ? $value
      : $this->uuidGenerator->generate();
  }

  protected function isUuid(string $value): bool {
    return preg_match($this->uuidPattern, $value) === 1;
  }

}
