<?php

declare(strict_types=1);

namespace Drupal\migrate_dc\Plugin\migrate\process;

use Drupal\Component\Utility\NestedArray;
use Drupal\migrate\Attribute\MigrateProcess;
use Drupal\migrate\MigrateException;
use Drupal\migrate\MigrateExecutableInterface;
use Drupal\migrate\ProcessPluginBase;
use Drupal\migrate\Row as MigrateRow;

#[MigrateProcess(
  id: 'migrate_dc_file_content',
)]
class MigrateDcFileContent extends ProcessPluginBase {

  /**
   * {@inheritDoc}
   *
   * @phpstan-param array<string, mixed> $configuration
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
  ) {
    $configuration += [
      'path' => '',
      'file_name' => '',
    ];

    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\migrate\MigrateException
   */
  public function transform($value, MigrateExecutableInterface $migrate_executable, MigrateRow $row, $destination_property) {
    $destination = $row->getDestination();

    $path = $this->getConfigValue('path', $destination);
    $fileName = $this->getConfigValue('file_name', $destination);
    $filePath = "$path/$fileName";

    try {
      $content = $this->fileGetContents($filePath);
    }
    catch (\RuntimeException) {
      $content = '';
    }

    return $content;
  }

  /**
   * @phpstan-param array<string, mixed> $destination
   *
   * @throws \Drupal\migrate\MigrateException
   */
  protected function getConfigValue(string $key, array $destination): string {
    $keyExists = FALSE;
    $value = $this->configuration[$key];
    if (mb_strpos($value, '@') === 0) {
      $value = NestedArray::getValue(
        $destination,
        explode(MigrateRow::PROPERTY_SEPARATOR, mb_substr($value, 1)),
        $keyExists,
      );
    }

    if (!$value) {
      throw new MigrateException("Config $key value does not exist", 1);
    }

    return $value;
  }

  protected function fileGetContents(string $fileName): string {
    $content = @file_get_contents($fileName);
    if ($content === FALSE) {
      throw new \RuntimeException("file '$fileName' count not be read", 1);
    }

    return $content;
  }

}
