<?php

declare(strict_types=1);

namespace Drupal\migrate_dc\Plugin\migrate\process;

use Drupal\Core\Password\PasswordInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\migrate\Attribute\MigrateProcess;
use Drupal\migrate\MigrateExecutableInterface;
use Drupal\migrate\ProcessPluginBase;
use Drupal\migrate\Row as MigrateRow;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Converts plain password to hashes.
 */
#[MigrateProcess(
  id: 'migrate_dc_plain_password',
)]
class MigrateDcPlainPassword extends ProcessPluginBase implements ContainerFactoryPluginInterface {

  protected string $defaultValue = 'a';

  /**
   * {@inheritDoc}
   *
   * @phpstan-param array<string, mixed> $configuration
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition,
  ): static {
    // @phpstan-ignore-next-line
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('password'),
    );
  }

  /**
   * {@inheritdoc}
   *
   * @phpstan-param array<string, mixed> $configuration
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    protected PasswordInterface $passwordChecker,
  ) {
    $configuration += ['default_value' => $this->defaultValue];

    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritDoc}
   */
  public function transform($value, MigrateExecutableInterface $migrate_executable, MigrateRow $row, $destination_property) {
    if (!$value) {
      $value = $this->configuration['default_value'];
    }

    return $this->passwordChecker->hash($value);
  }

}
