<?php

declare(strict_types=1);

namespace Drupal\Tests\migrate_dc\Kernel\Plugin\migrate\process;

use Drupal\migrate\MigrateExecutable;
use Drupal\migrate\MigrateMessageInterface;
use Drupal\migrate\Row as MigrateRow;
use Drupal\migrate_dc\Plugin\migrate\process\MigrateDcUriTransformator;
use Drupal\node\Entity\Node;
use Drupal\node\Entity\NodeType;
use Drupal\KernelTests\KernelTestBase;
use Drupal\node\NodeTypeInterface;
use PHPUnit\Framework\Attributes\CoversClass;
use PHPUnit\Framework\Attributes\Group;

#[Group('migrate_dc')]
#[CoversClass(MigrateDcUriTransformator::class)]
class MigrateDcUriTransformatorTest extends KernelTestBase implements MigrateMessageInterface {

  /**
   * {@inheritdoc}
   *
   * @var array<string>
   */
  protected static $modules = [
    'system',
    'field',
    'filter',
    'text',
    'user',
    'node',
    'migrate',
    'migrate_dc',
  ];

  protected string $nodeTypeId = 'page';

  protected NodeTypeInterface $nodeType;

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  protected function setUp(): void {
    parent::setUp();

    $values = [
      'type' => $this->nodeTypeId,
      'name' => 'Page',
    ];
    $this->nodeType = NodeType::create($values);
    $this->nodeType->save();

    $this->installEntitySchema('node');
    $this->installEntitySchema('user');
    $this->installConfig(static::$modules);
  }

  /**
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function testTransform(): void {
    $row = new MigrateRow();
    $destinationProperty = '';

    $container = \Drupal::getContainer();

    /** @var \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager */
    $entityTypeManager = $container->get('entity_type.manager');

    /** @var \Drupal\migrate\Plugin\MigrationPluginManagerInterface $migrationPluginManager */
    $migrationPluginManager = $container->get('plugin.manager.migration');

    $migration = $migrationPluginManager->createStubMigration([]);

    $migrateExecutable = new MigrateExecutable(
      $migration,
      // @todo Message instance.
      $this,
    );

    /** @var \Drupal\node\Entity\Node $node */
    $node = Node::create([
      'type' => $this->nodeType->bundle(),
      'title' => 'Test node',
      'uid' => 1,
    ]);
    $node->save();

    $pluginId = 'migrate_dc_uri_transformator';
    $pluginDefinition = [];
    $configuration = [];
    $expected = 'entity:node/' . $node->id();
    $value = 'entity.uuid:node/' . $node->uuid();

    $processor = new MigrateDcUriTransformator(
      $configuration,
      $pluginId,
      $pluginDefinition,
      $entityTypeManager,
    );

    $actual = $processor->transform(
      $value,
      $migrateExecutable,
      $row,
      $destinationProperty,
    );

    static::assertSame($expected, $actual);
  }

  /**
   * {@inheritdoc}
   */
  public function display($message, $type = 'status'): void {
    static::assertTrue($type === 'status', $message);
  }

}
