<?php

declare(strict_types=1);

namespace Drupal\Tests\migrate_dc\Unit\Plugin\migrate\process;

use Drupal\migrate\MigrateExecutable;
use Drupal\migrate\Row;
use Drupal\migrate_dc\Plugin\migrate\process\MigrateDcPlainPassword;
use Drupal\Tests\UnitTestCase;
use Drupal\Core\Password\PasswordInterface;
use PHPUnit\Framework\Attributes\CoversClass;
use PHPUnit\Framework\Attributes\DataProvider;
use PHPUnit\Framework\Attributes\Group;
use Symfony\Component\DependencyInjection\ContainerInterface;

#[Group('migrate_dc')]
#[CoversClass(MigrateDcPlainPassword::class)]
class MigrateDcPlainPasswordTest extends UnitTestCase {

  /**
   * @return array<string, array<string, mixed>>
   */
  public static function casesTransform(): array {
    return [
      'with value and config' => [
        'expected' => 'hash-of-value',
        'configuration' => [
          'default_value' => 'asdf',
        ],
        'value' => 'asd',
      ],
      'empty value with config' => [
        'expected' => 'hash-of-config-default-value',
        'configuration' => [
          'default_value' => 'qwer',
        ],
        'value' => '',
      ],
      'empty value without config' => [
        'expected' => 'hash-of-hardcoded-default-value',
        'configuration' => [],
        'value' => '',
      ],
    ];
  }

  /**
   * @phpstan-param array<string, mixed> $configuration
   */
  #[DataProvider('casesTransform')]
  public function testTransform(string $expected, array $configuration, string $value): void {
    $pluginId = '';
    $pluginDefinition = [];

    /** @var \Drupal\Core\Password\PasswordInterface|\PHPUnit\Framework\MockObject\MockObject $passwordChecker */
    $passwordChecker = $this->createMock(PasswordInterface::class);
    $map = [
      [
        $value,
        'hash-of-value',
      ],
      [
        'a',
        'hash-of-hardcoded-default-value',
      ],
    ];

    if (array_key_exists('default_value', $configuration)) {
      $map[] = [
        $configuration['default_value'],
        'hash-of-config-default-value',
      ];
    }

    $passwordChecker
      ->method('hash')
      ->willReturnMap($map);

    /** @var \Symfony\Component\DependencyInjection\ContainerInterface|\PHPUnit\Framework\MockObject\MockObject $container */
    $container = $this->createMock(ContainerInterface::class);
    $container
      ->expects(static::once())
      ->method('get')
      ->with('password')
      ->willReturn($passwordChecker);

    $processor = MigrateDcPlainPassword::create(
      $container,
      $configuration,
      $pluginId,
      $pluginDefinition,
    );

    /** @var \Drupal\migrate\MigrateExecutable $migrateExecutable */
    $migrateExecutable = $this->createMock(MigrateExecutable::class);
    $row = new Row();
    $destinationProperty = '';

    static::assertSame(
      $expected,
      $processor->transform(
        $value,
        $migrateExecutable,
        $row,
        $destinationProperty,
      )
    );
  }

}
