<?php

declare(strict_types=1);

namespace Drupal\Tests\migrate_dc\Unit\Plugin\migrate\process;

use Drupal\commerce_price\Price;
use Drupal\commerce_shipping\ShipmentItem;
use Drupal\migrate\MigrateExecutable;
use Drupal\migrate\Row as MigrateRow;
use Drupal\migrate_dc\Plugin\migrate\process\MigrateDcShipmentItem;
use Drupal\physical\Weight;
use Drupal\physical\WeightUnit;
use Drupal\Tests\UnitTestCase;
use PHPUnit\Framework\Attributes\CoversClass;
use PHPUnit\Framework\Attributes\DataProvider;
use PHPUnit\Framework\Attributes\Group;

#[Group('migrate_dc')]
#[CoversClass(MigrateDcShipmentItem::class)]
class MigrateDcShipmentItemTest extends UnitTestCase {

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    if (!class_exists(ShipmentItem::class)) {
      static::markTestIncomplete();
    }

    parent::setUp();
  }

  /**
   * @return array<string, array<string, mixed>>
   */
  public static function casesTransform(): array {
    return [
      'basic' => [
        'expected' => [
          new ShipmentItem([
            'order_item_id' => '32',
            'title' => 'a',
            'quantity' => 1,
            'weight' => new Weight('2', WeightUnit::KILOGRAM),
            'declared_value' => new Price('10', 'USD'),
          ]),
        ],
        'value' => [
          [
            'order_item_id' => '32',
            'title' => 'a',
            'quantity' => 1,
            'weight' => [
              'number' => 2,
              'unit' => WeightUnit::KILOGRAM,
            ],
            'declared_value' => [
              'number' => '10',
              'currency_code' => 'USD',
            ],
          ],
        ],
      ],
    ];
  }

  /**
   * @param array<\Drupal\commerce_shipping\ShipmentItem> $expected
   * @param array<mixed> $value
   */
  #[DataProvider('casesTransform')]
  public function testTransform(array $expected, array $value): void {
    $pluginId = '';
    $pluginDefinition = [];
    $configuration = [];

    /** @var \Drupal\migrate\MigrateExecutable $migrateExecutable */
    $migrateExecutable = $this->createMock(MigrateExecutable::class);
    $row = new MigrateRow();
    $destinationProperty = '';

    $migrateShipmentItem = new MigrateDcShipmentItem(
      $configuration,
      $pluginId,
      $pluginDefinition,
    );

    static::assertEquals(
      $expected,
      $migrateShipmentItem->transform(
        $value,
        $migrateExecutable,
        $row,
        $destinationProperty,
      )
    );
  }

}
